/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2003  Matrix project
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.AVLTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;

public class AVL_Insert implements SimulationExerciseModel, ButtonExercise, ModelAnswerNames, ButtonGroups, ConfigureVisualType  {
    private Table t;
    private ExerAVL ex;
    private AVLTree avl;
    private String S;
    public static final boolean DEBUG = false;

    String marker = "AVL_Insert";

    public AVL_Insert() {
    }

    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {

        // Create new AVL tree
        ex=new ExerAVL();
        boolean singleNeeded = false;
        boolean doubleNeeded = false;
        Random R = new Random(seed);
        do {
            // 15-20 keys
            int n = Math.abs(R.nextInt()%4);
            // Can contain duplicates
            S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,10)+
                matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);
            t = new Table(S);
            // First 6 should have single rotation
            for(int i = 0; i < 6; i++) {
                ex.realInsert((t.getObject(i)));
                if (ex.singleWasNeeded() && !singleNeeded)
                    singleNeeded=true;
                if (ex.doubleWasNeeded() && !doubleNeeded)
                    doubleNeeded=true;
            }
            // After that, there should be double rotation
            for(int i = 6; i < S.length(); i++) {
                ex.realInsert(t.getObject(i));
                if (ex.doubleWasNeeded() && !doubleNeeded)
                    doubleNeeded=true;
            }
            if (DEBUG)
                System.out.println("iteration");
        } while(singleNeeded == false || doubleNeeded == false);


        ex = new ExerAVL();
        FDT[] s = {t, ex};
        return s;
    }

    public String[] getStructureNames() {
        String[] s = { "Stream of Keys", "AVL Tree" };
        return s;
    }

    public String getDescription() {
        return "";
        //"Insert the following Keys into an initially empty AVL Tree";
    }

    public String[] buttonNames() {
        String[] s = {"Single rotation left", "Single rotation right",
                      "LR Double rotation ", "RL Double rotation"};
        return s;
    }

    public String[] buttonCommands() {
        String[] s = {"reflectSelectedVisualType(reflectEDT(SingleRotationLeft))",
                      "reflectSelectedVisualType(reflectEDT(SingleRotationRight))",
                      "reflectSelectedVisualType(reflectEDT(DoubleRotationLeft))",
                      "reflectSelectedVisualType(reflectEDT(DoubleRotationRight))"};
        return s;
    }


    public int[] buttonGroups() {
        int[] n = new int[1];
        n[0] = 2;
        return n;
    }



    public String[] getModelAnswerNames() {
        String[] s = { "AVL Tree" };
        return s;
    }



  public SimulationExerciseModel getModelAnswer() {
    return this;
  }

    public FDT[] solve() {
        Note.out(this, "Solve");
        avl = new AVLTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            avl.insert(tbl.getObject(i));
            a.endOperation();
        }
        Note.out(this, "Solved");
        return new FDT[] {avl };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerAVL() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, ex };
    }

    public FDT[] getAnswer() {
        FDT[] s = {ex };
        return s;
    }


    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }

    static final long serialVersionUID = -8436352103244731304L;
}
