/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */





package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.BinSearchTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;
import matrix.util.Note;

public class BST_Delete implements SimulationExerciseModel, ButtonExercise, ModelAnswerNames, ConfigureVisualType {
    final static int MAARA=4;
    Table t,table;
    BinSearchTree bst,bt2;
    String keys="";
    private String S = null;
    // model solution animator marker
    String marker = "BST_Delete";
    public static final boolean DEBUG = false;

    public BST_Delete() {
        keys = "";
    }

    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {

        ExerBin bt;
        keys = "";
        // Generate tree
        int largest;
        int smallest;
        Random R = new Random(seed);
        do {
            // Generate good BST tree

            // 15-20 keys
            int n = Math.abs(R.nextInt()%6);
            // Can contain duplicates
            S = //matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(15)+
                matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,15+n);
            // Maximum height 6

            t = new Table(S);
            //bt = new ExerBin();

            bt = new ExerBin((new Key(""+S.charAt(0))));

            // Insert keys
            for (int i=1; i <S.length(); i++)
                bt.realInsert((Element)new Key(""+S.charAt(i)));

            // Calc heights
            largest=bt.getLargestHeight();
            smallest=bt.getMinimumHeight();
            if (DEBUG)
                System.out.println("iteration "+smallest+":"+largest);

        } while (smallest < 3 || largest > 6);

        String newS= new String();
        String sep="";
        for (int i=0; i <MAARA; i++, sep=" ,") {
            char c =S.charAt(((MAARA-i-1)*S.length())/MAARA);
            newS+=""+ c;
            keys+= sep + c;
        }

        //keys = newS;
        table = new Table(newS);
        bt2 = new BinSearchTree();
        bt2.insert(t);
        FDT[]  s = { bt2 };
        return s;
    }



    public String[] getStructureNames() {
        String[] s =  { "Binary Search Tree" };
        return s;
    }

    public String getDescription() {
        return "Remove the following Keys: "+keys;
    }



    public String[] buttonNames() {
        String[] s = {"Update References"};
        return s;
    }

    public String[] buttonCommands() {
        String[] s = {"updateReferences"};
        return s;
    }

    public SimulationExerciseModel getModelAnswer() {
      return this;
    }

    public FDT[] solve() {
        Note.out(this, "Solve");
        bst = (BinSearchTree)getInitialStructures()[0];//new BinSearchTree();
        Animator a = Animator.getActiveAnimator();
        //	for (int i=0; i<t.size(); i++) {
        //  bst.insert(t.getObject(i));
        // }

        for (int i=0; i<MAARA; i++) {
            a.startOperation();
            bst.delete(table.getObject(i));
            a.endOperation();
        }


        Note.out(this, "Solved");
        return new FDT[] { (FDT)bst.getElement() };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public String[] getModelAnswerNames() {
        String[] s = { "Binary Search Tree" };
        return s;
    }

    public FDT[] getInitialStructures() {
        Table tbl = new Table(S);
        BinSearchTree ex = new BinSearchTree();
        ex.insert(tbl);
        return new FDT[] { ex };
    }

    public FDT[] getAnswer() {
        FDT[] s = { bt2 };
        return s;
    }


    public VisualTypeConf[] conf() {

        VisualTypeConf tre = new VisualTypeConf();

        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);

        tre.enable("matrix.visual.VisualLayeredTreeComponent",
           VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualReference", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualReference", VisualTypeConf.DRAG_OPERATION);

        return new VisualTypeConf[] { tre};
    }

    static final long serialVersionUID = -3394028505321162930L;
}
