/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.util.*;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.probe.*;
import matrix.util.*;

public class BST_TraversePre implements SimulationExerciseModel, StyledExercise, ModelAnswerNames, ConfigureVisualType {

  BinSearchTree bst = null;
  LinkedList l1;
  public static final boolean DEBUG = false;
    private String keys;



    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {
        bst = new BinSearchTree();
        keys = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed), 10);
        Table t = new Table(keys);
        bst.insert(t);
        l1 = new QueueImpl();
        FDT[] tmp = {bst, l1 };
        return tmp;
    }

    public String[] getStructureVisualisations() {
        return new String[] { "layered tree", "list" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "list"  };
    }


  public String[] getStructureNames() {
    String[] tmp = { "Tree", "preorder" };
    return tmp;
  }

  public String getDescription() {
    return "";
  }

  public FDT[] solve() {
    LinkedList sl1, last1;
    FDT[] fd = getInitialStructures();
    sl1 = (LinkedList)fd[1];//new LinkedListImpl();
    last1 = sl1;
    //    Table tbl = new Table(keys);
    BinaryTree bt = (BinaryTree)((BinSearchTree)fd[0]).getElement();//(BinaryTree) bst.getElement();
    StringTokenizer pre = new StringTokenizer(Tranverse.tranversePreOrder(bt));
    String tmp;
    Animator a = Animator.getActiveAnimator();
    while(pre.hasMoreTokens()) {
      a.startOperation();
      try {
        tmp = pre.nextToken();
        last1.setNext(last1.getNewNode(tmp));
        last1 = last1.getNext();
      }catch(NoSuchElementException e) {
        Note.err(this, "Error during solve, one StringTokenizer shorter than pre");
      }
      a.endOperation();
    }
    FDT[] ans = { sl1.getNext() };
    return ans;
  }



    public FDT[] makeModelAnswer() {
	return solve();
    }


  public String[] getModelAnswerNames() {
    String[] s = { "Binary Search Tree" };
    return s;
  }

  public FDT[] getInitialStructures() {
    Table tbl = new Table(keys);
    BinSearchTree tr = new BinSearchTree();
    tr.insert(tbl);
    return new FDT[] { tr, new QueueImpl() };
  }

  public FDT[] getSimulatedStructures() {
    return new FDT[] { bst, l1 };
  }


  public FDT[] getAnswer() {
    FDT[] tmp = {l1 };
    return tmp;
  }

  public SimulationExerciseModel getModelAnswer() {
    return null;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tre = new VisualTypeConf();
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    return new VisualTypeConf[] { tre, lis};
  }
  static final long serialVersionUID = -2641400449334300951L;
}
