/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: BinarySearch.java,v 1.27 2004/05/26 08:05:41 vkaravir Exp $ */

package content.exercises;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.QueueImpl;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;
import matrix.util.Note;

public class BinarySearch implements SimulationExerciseModel,
                                     StyledExercise, ModelAnswerNames, ConfigureVisualType {
  private VirtualArray user;
  private LinkedList userList;
  protected int keyToSearch = 0;
  protected int[] keyArray = null;
  protected String keyString = null;
  private LinkedList modelList;
  private VirtualArray model;
  public static final boolean DEBUG = false;

  public BinarySearch() {
    //charKeys = new char[(int)'Z' - (int)'A' + 1];
    //for(char i='A'; i<='Z'; i++) {
    //charKeys[i-'A'] = i;
    //}
  }

  public BinarySearch(int[] sortedKeys, int key) {
    keyArray=sortedKeys;
    keyToSearch = key;
  }


    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



  public FDT[] init() {
    java.util.Random r =  new java.util.Random(seed);
    keyArray = matrix.util.RandomKey.createNoDuplicateSortedIntKeys(r,200,400,30);
    boolean ok = false;
    keyString = "" + keyArray[20];
    for (int i = 21; i < keyArray.length; i++)
	keyString += "," + keyArray[i];
    // Select keyToSearch to be between the 20th and the last key in array.
    // The array shoudn't contain keyToSearch.
    while (!ok) {
      keyToSearch = matrix.util.RandomKey.createIntKey(r,keyArray[20],keyArray[keyArray.length-1]);
      ok = (keyString.indexOf("" + keyToSearch) == -1);
    } 
    user = new Table(keyArray);
    userList = new QueueImpl();

    FDT[]  s = { user, userList };
    return s;
  }

  public String[] getStructureNames() {
    String[] s = { "Table of Keys","List of keys visited in binary search"};
    return s;
  }

  public String[] getStructureVisualisations() {
    String[] s = { "array", "list" };
    return s;
  }


  public String[] getModelAnswerVisualisations() {
    String[] s = { "list" };
    return s;
  }



  public String getDescription() {
      return "Key to find: " +keyToSearch;

  }

  public String[] getModelAnswerNames() {
    String[] s = { "Table of Keys" };
    return s;
  }

  public FDT[] getAnswer() {
    FDT[] s = { userList };
    return s;
  }


  public FDT[] init(Object input) {
    if(input instanceof Character)
      keyToSearch = ((Character)input).charValue();
    return init();
  }

  public FDT[] solve() {
    Note.out(this, "Solve");

    Element[] keys = new Element[(int)'Z' - (int)'A' + 1];
    model = (VirtualArray)getInitialStructures()[0];
    modelList = (LinkedList)getInitialStructures()[1];
    LinkedList last = modelList;

    Animator a = Animator.getActiveAnimator();
    int low = 0;
    int high = model.size()-1;
    int mid = 0;
    while (low <= high) {
      a.startOperation();
      mid = getMiddleOne(low,high);
      last.setNext(last.getNewNode(model.getObject(mid)));
      last = last.getNext();
      try {
        if( keyArray[mid] < keyToSearch)
          low = mid + 1;
        else if( keyArray[ mid ] > keyToSearch )
          high = mid - 1;
        else {
          a.endOperation();
          return new FDT[] { modelList.getNext() };  /* Found mid*/
        }
      }  catch(ArrayIndexOutOfBoundsException e) {
        Note.out(this,e+"index = "+mid);
        a.endOperation();
        return new FDT[] { modelList.getNext() };
      }
      a.endOperation();
    }


    Note.out(this, "Solved");
    return new FDT[] { modelList.getNext() };
  }


    public FDT[] makeModelAnswer() {
	return solve();
    }



  public FDT[] getInitialStructures() {
    return new FDT[] { new Table(keyArray), new QueueImpl() };
  }

  public FDT[] getSimulatedStructures() {
    return new FDT[] { user, userList };
  }

  protected int getMiddleOne(int left, int right) {
    //Note.out(this,"calc middle one");
    return  ( left + right ) / 2;
  }

  public SimulationExerciseModel getModelAnswer() {
    return this;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tabl = new VisualTypeConf();
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    //lis.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
    //lis.enable("matrix.visual.VisualListComponent", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    return new VisualTypeConf[] { tabl, lis};
  }

  static final long serialVersionUID = -3331559942845572755L;
}
