/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: DFS.java,v 1.23 2004/05/26 08:05:41 vkaravir Exp $ */

package content.exercises;

import java.awt.Color;
import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.DirectedGraphImpl;
import matrix.structures.memory.Key;
import matrix.util.Note;

public class DFS implements SimulationExerciseModel,StyledExercise, ButtonExercise, ModelAnswerNames, ConfigureVisualType {
    private DirectedGraphImpl user;
    private DirectedGraphImpl model;
    private final String keys = "ABCDEFGHI";
    private int randomWanted = 3;
    private int randomTries = 100;
    private ExerVertex ModelVertex[];
    private String[] NodeList;
    public static final boolean DEBUG = true;

    public DFS() {

    }

    
    public VisualTypeConf[] conf() {
        VisualTypeConf graph = new VisualTypeConf();
        graph.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);        	
        return new VisualTypeConf[] { graph };
    }


    // styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "kamada-kawai graph" };
        return s;
    }


    public String[] getModelAnswerVisualisations() {
        String[] s = { "kamada-kawai graph" };
        return s;
  }


    public String[] buttonNames() {
        String[] s = {"Mark Visited","Mark Finished"};
        return s;
    }

    public String[] buttonCommands() {
        String[] s = {"reflectSelectedVisualType(reflectEDT(markVisited))",
                      "reflectSelectedVisualType(reflectEDT(markFinished))"};

        return s;
    }



    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {

        int l = keys.length();
        Random R = new Random(seed);
        int i,n;


        ExerVertex V[] = new ExerVertex[l];
        ModelVertex = new ExerVertex[l];
        NodeList = new String[l];
        user = new DirectedGraphImpl();
        model = new DirectedGraphImpl();

        // Create vertices
        for(i=0; i<l; i++) {
            V[i]=new ExerVertex();
            V[i].setElement(new Key(""+keys.charAt(i)));
            ModelVertex[i]=new ExerVertex();
            ModelVertex[i].setElement(new Key(""+keys.charAt(i)));
            NodeList[i] = "" + V[i].getElement() + ": ";
        }

        // Path from root node to every node
        for(i=1; i<l; i++) {
            n = Math.abs(R.nextInt()%i);
            V[n].addSuccessor(V[i]);
            ModelVertex[n].addSuccessor(ModelVertex[i]);
            NodeList[n] += V[i].getElement() + " ";
            V[i].addSuccessor(V[n]);
            ModelVertex[i].addSuccessor(ModelVertex[n]);
            NodeList[i] += V[n].getElement() + " ";
        }

        // Add random nodes
	int randomCreated = 0;
        for(i=0; i<randomTries; i++) {
            n = Math.abs(R.nextInt()%l);
	    if (DEBUG)
		System.out.println("i:"+i%l+" n:"+n);
	    if ((!V[i%l].hasSuccessor(V[n])) && (i%l != n)) {
		V[i%l].addSuccessor(V[n]);
		NodeList[i%l] += V[n].getElement() + " ";
		ModelVertex[i%l].addSuccessor(ModelVertex[n]);
	       
		V[n].addSuccessor(V[i%l]);
		NodeList[n] += V[i%l].getElement() + " ";
		ModelVertex[n].addSuccessor(ModelVertex[i%l]);
		randomCreated++;
		if (randomCreated >= randomWanted) break; 
	    } 
        }
	
        user.setVertices(V);
        model.setVertices(ModelVertex);
        FDT[]  s = { user };
        return s;
    }

    public FDT[] init(Object input) {
        return init();
    }


    public String[] getStructureNames() {
        String[] s = { "Graph" };
        return s;
    }


    public String getDescription() {
        String S="Click the nodes in DFS order."
            + "Start with node:" + keys.charAt(0) +"\n\n";
        for (int i=0; i<keys.length(); i++)
            S+=NodeList[i]+"\n";

        return S;
    }



    private void depthFirstSearch(ExerVertex v) {
        // If already visited, terminate
        if (v.visited() || v.finished()) return;

        Animator a = Animator.getActiveAnimator();

        // Mark visited
        a.startOperation();

        if (DEBUG) 
	    System.out.println("visited:"+v.getElement());
        v.markVisited();
        
        a.endOperation();
        // Travel through successorlist
        Vertex[] vertices = v.getSuccessors();
        for (int i=0; i<vertices.length; i++)
            depthFirstSearch((ExerVertex)vertices[i]);

        a = Animator.getActiveAnimator();

        // Mark finished
        a.startOperation();
	if (DEBUG)
	    System.out.println("finished:"+v.getElement());
        v.markFinished();
        a.endOperation();
    }


    public FDT[] solve() {
        Note.out(this, "Solve");

        Animator a = Animator.getActiveAnimator();


        //a.startOperation();
        Vertex[] vertices = model.getVertices();
        for (int i = 0; i < vertices.length; i++) {
            ((ExerVertex)vertices[i]).setColor(Color.white);
        }
        //a.endOperation();

        // a.startOperation();
        depthFirstSearch(ModelVertex[0]);
        // a.endOperation();

        //traverse.traverse(model);

        Note.out(this, "Solved");
        return new FDT[] { model };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public String[] getModelAnswerNames() {
        String[] s = { "Graph" };
        return s;
    }

    public FDT[] getAnswer() {
        FDT[] s = { user };
        return s;
    }

    public FDT[] getInitialStructures() {
        throw new RuntimeException("not implemented");
    }

    static final long serialVersionUID = 6124519500007486392L;
}
