/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: ExerHeap.java,v 1.17 2004/05/26 08:05:42 vkaravir Exp $ */

package content.exercises;

import matrix.structures.EDT.EDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.*;
import matrix.util.Note;

/**
 * This is an implementation of a binary tree. A binary tree is a
 * tree in which each node has two children (either of which can be null).
 */
public class ExerHeap extends BinTrei implements EDT {

    public static final boolean DEBUG = false;
    public static final String EMPTY = " ";
    private int keys = 0;

    public ExerHeap(int n) {
        super(n);
    }


    public ExerHeap(Object[] objs) {
        super(objs);
    }


    // Private constructor for compatibility with BinaryTree
    private ExerHeap(VirtualArray bt, int curr) {
        this.bt = bt;
        this.curr = curr;
    }



    /**
     * Constructs a heap in a bottom-up
     * manner to convert an array to heap
     */
    /*
       public void BuildHeap() {
       VirtulArray bt = (VirtualArray)this.getArray();
       for (int i =bt.getLast()/2; i >=0; i--)
       bubbleDown(i);
       }
    */

    public String[] getAllowedMethods() {
        return new String[]{"deleteRoot"};
    }


    public int leftChild(int n) {
        if(n == 0) return 1;
        return 2 * n + 1;
    }

    public int rightChild(int n) {
        if(n == 0) return 2;
        return 2 * n + 2;
    }

    public int parent(int n) {
        if (n == 0) return 0;
        return (n-1)/2;
    }

    public void Heapify(int i) {
        Note.out(this, "Heapify: " + i);
        Note.out(this, "lc: " + leftChild(i) + " rc: " + rightChild(i));
        if(i >= getSize())
            return;
        Element p = (Element)getObject(i);
        Element lc = (Element)getObject(leftChild(i));
        Element rc = (Element)getObject(rightChild(i));
        if(lc == null)
            return;
        if(rc == null || rc.equals(EMPTY)) { //LISTTY rc.equals empty OS
            if(p.gt(lc) && !lc.equals(EMPTY)) {
                swap(i, leftChild(i));
            }
            return;
        }
        if(lc.gt(rc)) {
            if(p.gt(rc) && !rc.equals(EMPTY)) {
                swap(i, rightChild(i));
                Heapify(rightChild(i));
            }
        } else {
            if(p.gt(lc) && !lc.equals(EMPTY)) {
                swap(i, leftChild(i));
                Heapify(leftChild(i));
            }
        }
    }

    public void insert(Object o) {
        if (DEBUG)
            System.out.println("Size:" + keys);

        setObject(o, keys++);
        bubble(keys-1);

    }

    public FDT delete(Object o) {
        Element tmp = (Element)getObject(keys - 1);
        setObject(tmp, 0);
        setObject(new Key(EMPTY), keys - 1);
        keys--;
        Heapify(0);
        return this;
    }

    /**
     * Empties the root node.
     * Does not "bubble" or swap.
     */
    public void deleteRoot() {
        setElement(new Key(EMPTY));
        /*	Key k = new Key(" ");
                setObject(k, 0);*/
    }

    BinaryTree left = null;
    public BinaryTree getLeft() {
        BinaryTree nbt = (left == null) ? left = new ExerHeap(bt, leftChild(curr)) : left;
        return (nbt.getElement() == null) ? null : nbt;
    }

    BinaryTree right = null;
    public BinaryTree getRight() {
        BinaryTree nbt = (right == null) ? right = new ExerHeap(bt, rightChild(curr)) : right;
        return (nbt.getElement() == null) ? null : nbt;
    }

    private void bubble(int n) {
        Element p = (Element)getObject(parent(n));
        Element t = (Element)getObject(n);

        if (p == null || p.gt(t)) {
            swap(n,parent(n));
            bubble(parent(n));
        }
    }

    static final long serialVersionUID = 141322807213670974L;
}
