/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: RBT_Insert.java,v 1.27 2004/05/26 08:05:45 vkaravir Exp $ */

package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.RBTree;
import matrix.structures.EDT.ExerRBTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;
//import ptn.*;

public class RBT_Insert implements SimulationExerciseModel, ButtonExercise, ButtonGroups, ModelAnswerNames, ConfigureVisualType {
    Table t;
    ExerRBTree user;
    RBTree model;
    String S = null;
    public static final boolean DEBUG = false;

    public RBT_Insert() {
    }

    public RBT_Insert(String initTable) { t = new Table(initTable); }



    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public FDT[] init() {

        // 9-13 keys
        Random R = new Random(seed);
        int n = Math.abs(R.nextInt()%4);
        // Can contain duplicates
        S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,9)+
            matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);

        t = new Table(S);
        user = new ExerRBTree();
        FDT[]  s = { t, user };
        return s;
    }



    public String[] getStructureNames() {
        String[] s = { "Stream of Keys", "Red Black Tree" };
        return s;
    }


    public String getDescription() {
        return "";
    }

    public String[] buttonNames() {
        String[] s = {"Toggle color", "Single rotation left", "Single rotation right",
                      "LR Double rotation", "RL Double rotation"};
        return s;
    }

    public String[] buttonCommands() {
        return new String[] {"reflectSelectedVisualType(reflectEDT(toggleColor))",
                             "reflectSelectedVisualType(reflectEDT(rotateLeft))",
                             "reflectSelectedVisualType(reflectEDT(rotateRight))",
                             "reflectSelectedVisualType(reflectEDT(doubleRotateLeft))",
                             "reflectSelectedVisualType(reflectEDT(doubleRotateRight))"};
    }


    public int[] buttonGroups() {
        int[] n = new int[2];
        n[0] = 1;
        n[1] = 3;
        return n;
    }


    public FDT[] solve() {
        Note.out(this, "Solve");
        model = new RBTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            model = (RBTree)model.insert(tbl.getObject(i));
            a.endOperation();
        }
        Note.out(this, "Solved");
        return new FDT[] { model };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerRBTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, user };
    }

    public FDT[] getAnswer() {
        FDT[] s = { user };
        return s;
    }

    public String[] getModelAnswerNames() {
        String[] s = { "Red-Black Tree" };
        return s;
    }


    //public SimulationExerciseModel getModelAnswer() {
    //return null;
    //}


    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }


    static final long serialVersionUID = -7576022670921264541L;
}
