/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: SimulationExercise.java,v 1.10 2004/05/25 13:28:01 kstalnac Exp $*/

package content.exercises;

import matrix.structures.FDT.FDT;

/**
 * This is the interface for all exercises that feature some sort of
 * data structure manipulation, algorithm animation or both.
 * In this interface are included method for retrieving the
 * user's answer to the exercise. Methods for creating model answers
 * or checking answers submitted by users are <em>NOT</em> included.
 *
 * @author Jussi Nikander
 * @version $Revision: 1.10 $
 */

public interface SimulationExercise extends Exercise {

    /** The data structures used in this exercise are initialized
     * in this method. The creation of the input of the exercise
     * can be handled here, if desired. The input could also,
     * for example, be given in the class constructor. The FDTs
     * returned by this method must be in the state they should be
     * before the user starts solving the exercise.
     *
     * @return an FDT array containing all FDTs used in this exercise.
     */
    public FDT[] init();



    public void setSeed(long seed);

    public long getSeed();


    /** Returns the names of the data structures returned by
     * <code>init()</code>. The names in the array returned by this
     * method must be in the same order as FDTs are in the array
     * returned by <code>init()</code>.
     *
     * @return a String array containing the names of the FDTs used
     * in this exercise
     */
    public  String[] getStructureNames();


    /**
     * Returns the user's answers to the exercise for judging.
     * This method returns those FDTs that are required for
     * checking this exercise. The FDTs in the array must be
     * in the same order than the FDTs returned by the
     * <code>solve</code> -method of <code>SimulationExerciseModel</code>
     * if such method is present.
     *
     * @return the FDTs that are part of the answer for this exercise.
     */
    public FDT[] getAnswer();


    /**
     * Returns a copy of all the FDTs in the exercise.  User's anwers
     * can be checked against a model solution that is produced by
     * applying the exercise's algorithm to the initial values of the
     * exercise.<p>
     *
     * The method is quarenteered always to return new FDTs that are
     * in the initial state.
     *
     * @return the FDTs that are the initial values of the exercise.
     */
    public FDT[] getInitialStructures();



    static final long serialVersionUID = 6258117896522976L;
}
