/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: SimulationExerciseModel.java,v 1.8 2004/05/25 13:28:01 kstalnac Exp $*/

package content.exercises;

import matrix.structures.FDT.FDT;

/**
 * This interface contains methods that are required for creating a model
 * answer for a <code>SimulationExercise</code>. The model answer can
 * either be used as a model to be show to the user or in checking and
 * grading an user solution.
 * <p>
 * The making and solving of exercises and the creation of model answers
 * is separated into two interfaces in order to allow the creation of
 * a server / client model, where exercises are sent to clients, and
 * answers are later graded at the server.
 *
 * @author Jussi Nikander
 * @version
 */
public interface SimulationExerciseModel extends SimulationExercise {
    /**
     * Initialises the structures needed for model answer and
     * solves the exercise, returning a model answer of the
     * exercise. The FDTs returned by solve must be in the
     * same order as the FDTs returned by <code>getAnswer()</code>
     * in <code>SimulationExercise</code>
     *
     * @see #getAnimator()
     */
    public FDT[] solve();


    /**
     * The model solution that is used in the model solution window
     * could  be generated using the solve -method. In some cases, though,
     * some additional decorations are needed in the model solution animation.
     * Because of this the model solution animation is defined in different method. 
     * By default, makeModelAnswer can be implemented
     * <pre>
     * public FDT[] makeModelAnswer() {
     *   return solve();
     * }
     *</pre>
     *
     */
    public FDT[] makeModelAnswer();


  /**
   * Initializes the structures used in the model answer with
   * the given input.
   */
  //public FDT[] init(Object input);

  /* XXX init(Object) should probably be in SimulationExercise.
   * It is here in order to avoid writing dummy inits for all
   * test exercises that implement SimulationExercise.
   */


    static final long serialVersionUID = 5456218560520880004L;
}
