/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: StructureChangeOperation.java,v 1.3 2004/05/25 13:28:04 kstalnac Exp $ */

package matrix.animation;

import matrix.structures.FDT.FDT;
import matrix.util.Note;
import matrix.visual.*;

/**
 * AnimatedOperation that should be used when the structure changes.
 */
public class StructureChangeOperation implements AnimatedOperation {
    public FDT oldValue, newValue;

    public StructureChangeOperation(FDT oldValue, FDT newValue) {
        this.oldValue = oldValue;
        this.newValue = newValue;
    }

    /**
     * Undo one step in animation. The oldValue is set for all visualizations
     * (or more precisely for the container they're in) that represent the
     * FDT newValue.
     */
    public void undo() {
	VisualType[] visualizations = VisualType.getVisualizations(newValue);
	if (visualizations == null || visualizations.length == 0) {
	    Note.out(this, "Coudn't undo operation! No visualizations found");
	    return;
	}
	for (int i=0; i < visualizations.length; i++) {
	    while (visualizations[i].getParent() instanceof VisualType && !(visualizations[i] instanceof VisualContainer))
		visualizations[i] = (VisualType)visualizations[i].getParent();
	    visualizations[i].setStructure(oldValue);
	}
    }

    /**
     * Redo one step in animation. The newValue is set for all visualizations
     * (or more precisely for the container they're in) that represent the
     * FDT oldValue.
     */
     public void redo() {
	VisualType[] visualizations = VisualType.getVisualizations(oldValue);
	if (visualizations == null || visualizations.length == 0) {
	    Note.out(this, "Coudn't undo operation! No visualizations found");
	    return;
	}
	for (int i=0; i < visualizations.length; i++) {
	    while (visualizations[i].getParent() instanceof VisualType && !(visualizations[i] instanceof VisualContainer))
		visualizations[i] = (VisualType)visualizations[i].getParent();
	    visualizations[i].setStructure(newValue);
	}
    }

    static final long serialVersionUID = 5378791551507819489L;
}
