/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.decoration;


/**
 * @author Petri Tenhunen
 */
public interface InformationDecorator {

  /** 
   * Tells if labeling can be shown
   * @return true if label is shown
   */
  public boolean isInfoEnabled();
  
  /**
   * Sets the info to the current object
   * @param s the new label
   */
    public void setInfo(String s);
  
  /**
   * Returns the information assosiated with this object
   * @return the information
   */
  public String getInfo();
  
  /**
   * Returns the wait time in milliseconds. ToolTip is displayed
   * after this time is passed.
   * @return time to wait
   */
  public int getWaitTime();

  /** Set time to wait before ToolTip is displayed */
  public void setWaitTime(int t);

}
