/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: LabelDecorator.java,v 1.4 2004/05/25 13:28:06 kstalnac Exp $ */

package matrix.decoration;

/** 
 * The interface LaberDecorator is a hint for the visualizer to recognize 
 * that this particular node should be decorated by labeling its nodes
 * and/or references.
 *
 * @author Ari Korhonen
 * @version $Revision: 1.4 $
 */

public interface LabelDecorator {

    /** 
     * Tells if labeling is enabled for nodes.
     * @return true if label is shown
     */
    public boolean isLabelEnabled();
    /**
     * Sets the label.
     * @param s the new label
     */
    public void setLabel(String s);
    /**
     * Returns the label.
     * @return the label
     */
    public String getLabel();

    /**
     * Tells if labeling is enabled for references.
     * @return true if label is shown
     */
    public boolean isReferenceLabelEnabled();

    /**
     * Sets the label of the i'th reference.
     * @param i reference number
     * @param s the new label
     */
    public void setReferenceLabel(int i, String s);

    /**
     * Returns the i'th label.
     * @param i the reference number
     * @return the label
     */
    public String getReferenceLabel(int i);
}
