/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package matrix.simulation;

import java.util.Hashtable;

/**
   A class for configuring visual types.
   @author Panu Silvasti
 */
public class VisualTypeConf {

    public static final int DROP_OPERATION = 1;
    public static final int DRAG_OPERATION = 2;
    public static final int HIGHLIGHT_OPERATION = 4;
    public static final int POP_UP_MENU_OPERATION = 8;

    private Hashtable opHash = new Hashtable();
    

    /**
       Enables user interface operations for certain visual types.
     */
    public void enable(String visualTypeClass, int operation) {
	// XXX Should check that visualTypeClass exists!
	Object op = opHash.get(visualTypeClass);
	if (op == null) {
	    opHash.put(visualTypeClass, new Integer(operation));
	} else if ((((Integer)op).intValue() & operation)==0) {
	    opHash.put(visualTypeClass, new Integer(((Integer)op).intValue()+operation));
	}
    }
    
    
    /** 
	Is given operation enabled for given visual type class.
    */
    public boolean isEnabled(String visualTypeClass, int operation) {
	Object op = opHash.get(visualTypeClass);
	if (op == null) {
	    return false;
	} else {
	    int opInt = ((Integer)op).intValue();
	    // bitwise and operation
	    return (opInt & operation) != 0;
	}
    }
}
