/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: BinHeap.java,v 1.3 2004/05/26 08:05:49 vkaravir Exp $ */

package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.*;
import matrix.util.Note;

public class BinHeap extends BinTrei implements CDT {
    VirtualInteger n;

    public BinHeap() { n = new VirtualInteger(0); }

    public FDT insert(Object o) {
        Note.out(this, "INSERT "+o);

        if (o instanceof Element) {
            insert(n, (Element)o);
            return this;
        } else if (o instanceof Table) {
            //Note.show(this, "BuildHeap not implemented yet: doing insertions");
            Table t = (Table)o;
            for (int i=0; i<t.size(); i++)
                insert(t.getObject(i));
        } else Note.show(this, "Cannot insert object of type " + o.getClass());
        return this;
    }

    public FDT delete(Object o) {
        Element tmp = (Element)getObject(n.eval() - 1);
        setObject(tmp, 0);
        setObject(null, n.eval() - 1);
        n.dec();
        bubbledown(0);
        return this;
    }

    public Object search(Object o) { return null; }

    private void insert(VirtualInteger n, Element e) {
        setObject(e, n.eval());
        bubble(n.eval());
        n.inc();
    }

    public int leftChild(int n) {
        if(n == 0) return 1;
        return 2 * n + 1;
    }

    public int rightChild(int n) {
        if(n == 0) return 2;
        return 2 * n + 2;
    }

    public int parent(int n) {
        if (n == 0) return 0;
        return (n-1)/2;
    }

    private void bubbledown(int i) {
        Note.out(this, "Bubbledown: " + i);
        Note.out(this, "lc: " + leftChild(i) + " rc: " + rightChild(i));
        if(i == n.eval())
            return;
        Element p = (Element)getObject(i);
        Element lc = (Element)getObject(leftChild(i));
        Element rc = (Element)getObject(rightChild(i));
        if(lc == null)
            return;
        if(rc == null) {
            if(p.gt(lc)) {
                swap(i, leftChild(i));
            }
            return;
        }
        if(lc.gt(rc)) {
            if(p.gt(rc)) {
                swap(i, rightChild(i));
                bubbledown(rightChild(i));
            }
        } else {
            if(p.gt(lc)) {
                swap(i, leftChild(i));
                bubbledown(leftChild(i));
            }
        }
    }

    private void bubble(int n) {
        Element p = (Element)getObject(parent(n));
        Element t = (Element)getObject(n);
        if (p.gt(t)) {
            swap(n,parent(n));
            bubble(parent(n));
        }
    }

    static final long serialVersionUID = 8971892506567935031L;
}
