/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package matrix.structures.EDT;

import matrix.structures.FDT.FDT;

/**
 * Public interface for exercises. This datatype is used by a user when he solving the
 * exercise. The easiest way to create an EDT is to extend corresponding CDT and override
 * methods you want to hide from user. For example such a method might be the
 * <code>public FDT insert(Object o)</code>.
 *
 * @author Petri Tenhunen
 */
public interface EDT extends FDT {
    /**
     * <P>
     * Methods to be added to the visualizations pop-up menu. Only methods with no arguments
     * are allowed. If you have the method </p><p>
     * <code>public void testMethod();</code></p><p>
     * it can be added to the pop-up menu with followinf <code>getAllowedMethods()</code>
     * </p><p>
     * <code>return new String[] {"testMethod"};</code><p>
     *
     * @return table with method names
     */
    public String[] getAllowedMethods();

    static final long serialVersionUID = 6297448787279322449L;
}
