/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.EDT;
import java.awt.Color;

import matrix.decoration.*;
import matrix.structures.memory.*;
import matrix.util.Note;

/**
 *
 * @author Ari Korhonen
 * @author Panu Silvasti
 */

public class VisitableKey extends Key implements Visitable, Styled {

    private VirtualBoolean visited = new VirtualBoolean(false);
    protected StyleSheet ss =  null;
    
    public VisitableKey(String s) {
	super(s);
	initStyleSheet();
    }

    public VisitableKey(Object o) {
	super(o);
	initStyleSheet();
    }

    public boolean isVisited() {
	Note.out(this, "isVisited returns " + visited.eval());
	return visited.eval();
    }

    public void setVisited(boolean val) {
	Note.out(this, "setVisited = " + val);
	visited.assign(val);
    }



    public void initStyleSheet() {
	ss = new StyleSheetAdapter() {
		public Color getVisitedPenColor() { return Color.red; }
		public Color getVisitedBorderColor() { return getDefaultBorderColor();}
		
	    };
    }


    public StyleSheet getStyleSheet() {
	return ss;
    }


    public boolean equals(Key k) {
	if (k instanceof VisitableKey) {
	    if (visited==((VisitableKey)k).visited)
		return super.equals(k);
	    else
		return false;	    
	} else {
	    return false;
	}
    }
    public boolean equals(Object k) {
	if (k instanceof VisitableKey) {
	    if (visited==((VisitableKey)k).visited)
		return super.equals(k);
	    else
		return false;	    
	} else {
	    return false;
	}
    }


}
