/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package matrix.structures.FDT;

/**
 * Interfaces for dynamic vertices. The list of successors of a dynamic vertex
 * can be changed after creation.
 */

public interface SimulationVertex extends Vertex {

    /**
     * Sets the successors of this node.
     * @param v the new list of successors
     */
    public void setSuccessors(Vertex[] v);

    /**
     * Adds a successor to the list of successor vertices
     * @param v the new successor
     */
    public void addSuccessor(Vertex v);

    /**
     * Removes a successor from the list of successor vertices
     * @param v the successor to be removed
     */
    public void removeSuccessor(Vertex v);

    static final long serialVersionUID = -6519620791874161299L;
}
