/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: DirectedGraphImpl.java,v 1.4 2004/06/10 12:15:11 kstalnac Exp $ */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;
import matrix.util.Note;
import matrix.decoration.LabelDecorator;

/**
 * A very simple implementation of a graph. Cannot be visualised other than as a graph.
 * Supporst graph traversing algorithms.
 *
 * @author Jussi Nikander
 */

public class DirectedGraphImpl extends VirtualArray implements FDT, SimulationGraph, LabelDecorator {

    /** graph vertices are stored here */
    private VirtualArray vertices;

    private VirtualObject elem;


    public DirectedGraphImpl() {
        super();
        //vertices = new VirtualArray();
        vertices = this;
        elem = new VirtualObject();
    }

    public VertexImpl getNewVertex(Object o) {
	VertexImpl v = new VertexImpl(o);
	v.switchWeightEnabled(isReferenceLabelEnabled());
	return v;
	//return new VertexImpl(o);
    }

    // interface Traverse

    public void traverse() {
        //Depth_First_Search.dfs();
    }

    public boolean isVisited() {
        return false;
    }

    public void setVisited(boolean b) {
    }

    public String[] getTraversingMethods() {
        return new String[] { "DFS" };
    }

    public void setTraversingMethod(String s) {
    }

    // interface FDT

    public Object getElement() {
        if(elem.getObject() == null)
            return vertices.getObject(0);
        return elem.getObject();
    }

    public void setElement(Object o) {
        for(int i = 0; i < vertices.size(); i++) {
            if(o == vertices.getObject(i)) {
                elem.setObject(vertices.getObject(i));
                return;
            }
        }
    }

    // interface SimulationGraph

    public Vertex[] getVertices() {
        Vertex[] tmp = new Vertex[vertices.size()];
        for(int i = 0; i < vertices.size(); i++)
            tmp[i] = (Vertex)vertices.getObject(i);
        return tmp;
    }

    public void setVertices(Vertex[] v) {
        int i = 0;
        for(; i < v.length; i++)
            vertices.setObject(v[i], i);
        for(; i < vertices.size(); i++)
            vertices.setObject(null, i);
    }

    public void addVertex(Object o) {
        VertexImpl vi = null;
        if(o instanceof VertexImpl)
          vi = (VertexImpl)o;
        else if(o instanceof Element){
            vi = getNewVertex(o);
        }
        else if(o instanceof FDT) {
            vi = getNewVertex(new Key(((FDT)o).getElement()));
        }
        else
            vi = getNewVertex(new Key(o));
        for(int i =  0; i < vertices.size(); i++) {
          if(vertices.getObject(i) == null) {
            vertices.setObject(vi, i);
            return;
          }
        }
        vertices.setObject(vi, vertices.size());
    }

    public void removeVertex(Vertex v) {
        int i;
        Note.out(this, "deleting, size " + vertices.size());
        for(i = 0; i < vertices.size(); i++) {
          if(((Vertex)vertices.getObject(i)).equals(v))
                break;
        }
        for(;i < vertices.size(); i++)
            vertices.setObject(vertices.getObject(i + 1), i);
    }

    /*
    public void insertEdge(Vertex from, Vertex to) {
        if(from instanceof VertexImpl)
            ((VertexImpl)from).insertSuccessor(to);
        else
            Note.out(this, "cannot do insertEdge");
    }

    public void deleteEdge(Vertex from, Vertex to) {
        if(from instanceof VertexImpl)
            ((VertexImpl)from).deleteSuccessor(to);
        else
            Note.out(this, "Cannot delete edge");
    }
    */

    // Interface CDT
    /*
    public FDT insert(Object o) {
      Note.out(this, "Inserting...");

        if(o instanceof Table) {
            Table t = (Table)o;
            for(int i = 0; i < t.size(); i++)
                insert(t.getObject(i));
        }
        else {
            if(o instanceof VertexImpl)
              insertVertex((Vertex)o);
            else {
              VertexImpl tmp = getNewVertex(o);
              insertVertex(tmp);
            }
        }
        return this;
    }

    public FDT delete(Object o) {
        Note.show(this, "Delete by key value cannot be done to a graph node.");
        return this;
    }

    public Object search() {
        return this;
    }
    */

    // interface LabelDecorator
    private boolean isLabelEnabled = false;
    private boolean isReferenceLabelEnabled = false;
    private String label = "";
    private String referenceLabel = "";

    public boolean isLabeled() { return isLabelEnabled();}

    public boolean isLabelEnabled() { return isLabelEnabled;}
    public void setLabel(String s) {label = s;}
    public String getLabel() { return label;}
    public boolean isReferenceLabelEnabled() {
	return isReferenceLabelEnabled;
    }
    public void setReferenceLabel(int i, String s) {
	referenceLabel = s;
    }
    public String getReferenceLabel(int i) {
	return referenceLabel;
    }

    public boolean switchWeightEnabled(boolean weighted) {
	isReferenceLabelEnabled = weighted;

	for (int i = 0; i < vertices.size(); i++) {
	    ((VertexImpl)vertices.getObject(i)).switchWeightEnabled(isReferenceLabelEnabled);
	}

	return isReferenceLabelEnabled;
    }

    static final long serialVersionUID = -7993064623641268504L;
}
