/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.structures.FDT.probe;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;

/**
 * Implementation of a linked list. A linked list is a collection of
 * nodes that contain some information, and a pointer to the next node
 * in the list.
 *
 * @author Jussi Nikander
 * @version
 */
public class LinkedListImpl implements LinkedList {

    /** contents[0] is the key, contents[1] the successor */
    private VirtualArray contents;

    public LinkedListImpl() {
        contents = new VirtualArray();
    }

    public LinkedListImpl(Object key) {
        this();
        setElement(key);
    }

    public boolean equals(Object o) {
        if(o instanceof LinkedList)
            return Comparer.compareList(this, (LinkedList)o);
        else
            return false;
    }

    //--- interface FDT ----

    public Object getElement() {
        return contents.getObject(0);
    }

    public void setElement(Object o) {
        //if(o instanceof Key || o == null)
	if(o instanceof FDT || o == null)
	    contents.setObject(o, 0);
        else
            contents.setObject(new Key(o), 0);
    }

    // --- interface LinkedList ---

    public LinkedList getNext() {
        return (LinkedList) contents.getObject(1);
    }

    public void setNext(LinkedList l) {
        contents.setObject(l, 1);
    }

    public boolean hasNext() {
        return contents.size() == 2;
    }

    public LinkedList getNewNode(Object key) {
        return new LinkedListImpl(key);
    }

    public String toString() {
        return "key is: " + contents.getObject(0);
    }


    static final long serialVersionUID = -537868976419949417L;
}
