/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.Array;
import matrix.structures.memory.VirtualArray;
import matrix.util.Note;

public class Matrix implements Array {
    private VirtualArray array;
    private int n, m;

    public Matrix(int n, int m) {
        this.n = n;
        this.m = m;
        array = new VirtualArray();
        VirtualArray item;
        for (int i=0; i<n; i++) {
            array.setObject(item = new VirtualArray(), i);
            item.setFirst(0);
            item.setLast(m-1);
        }
    }

    // public interface

    public Object getObject(int n, int m) {
        VirtualArray va = (VirtualArray)array.getObject(n);
        return va.getObject(m);
    }

    public void setObject(int n, int m, Object o) {
        VirtualArray va = (VirtualArray)array.getObject(n);
        va.setObject(o, m);
    }

    // interface FDT

    public Object getElement() {
        return array;
    }
    public void setElement(Object o) {
        Note.err(this, "setElement not defined for FDT Matrix " + this);
    }

    // interface Array

    public Array getArray() {
        return array;
    }
    public Object getObject(int address) {
        return array.getObject(address);
    }
    public void setObject(Object o, int address) {
        Note.show(this, "Cannot change a whole row/column of Matrix");
    }
    public int getFirst() {
        return array.getFirst();
    }
    public int getLast() {
        return array.getLast();
    }
    public void setFirst(int i) {
        Note.err(this, "setFirst not defined for FDT Matrix " + this);
    }
    public void setLast(int i) {
        Note.err(this, "setFirst not defined for FDT Matrix " + this);
    }


    static final long serialVersionUID = -8996895218423328167L;
}
