/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.structures.adaptation;

import matrix.structures.FDT.*;
import matrix.util.Note;

public abstract class Adapter {

    public static LinkedList getStructure(StaticList sl) {
        ListNodeAdapter head = null;
        for (int i=sl.getMaxIndex(); i>=sl.getMinIndex(); i--) {
            ListNodeAdapter node = new ListNodeAdapter(sl.getKey(i));
            node.setNext(head);
            head = node;
        }
        return head;
    }

    private static TreeNodeAdapter[] T;
    public static Tree getStructure(StaticTree st) {
        int root = -1;
        int N = st.getNrOfNodes();
        T = new TreeNodeAdapter[N];
        int nrOfChildren[] = new int[N];
        // create nodes and set keys for them
        for (int i=0; i<N; i++) {
           T[i] = new TreeNodeAdapter(st.getKey(i));
           int father = st.getFather(i);
           if ((father != i) && (father >= 0) && (father < N))
               nrOfChildren[father]++;
           else if (father == i) {
               if (root == -1)
                   root = i;
               else
                   Note.show(null, "StaticTree with many roots: "+T[i]+" and "+T[root]);
           } else Note.err(null, "index (father) out of bounds error: "+father);
        }
        // set table of children for each node
        for (int i=0; i<N; i++) 
            T[i].setChildren(new TreeNodeAdapter[nrOfChildren[i]]);
        // add children for each node 
        for (int i=0; i<N; i++) 
            T[st.getFather(i)].addChild(T[i]);
        if (root != -1)
            return T[root];
        else {
            Note.show(null, "No root determined: setting index = 0 as root: "+T[0]);
            return T[0];
        }
    }

    private static GraphNodeAdapter[] G;
    private static Object label;
    public static Graph getStructure(StaticGraph sg) {
        G = new GraphNodeAdapter[sg.getNrOfNodes()];
        label = sg.getElement();
        // create vertices and set keys for them
        for (int i=0; i<sg.getNrOfNodes(); i++)
            G[i] = new GraphNodeAdapter(sg.getKey(i));
        // set successors of each vertex
        for (int i=0; i<sg.getNrOfNodes(); i++) {
            int[] succi = sg.getSuccessors(i);
            Vertex[] succv = new Vertex[succi.length];
            Note.out(null, "len="+succi.length);
            for (int j=0; j<succi.length; j++)
                succv[j] = G[succi[j]];
            G[i].setSuccessors(succv);
        }
        // create the graph including all the vertices
        return new Graph() {
                private GraphNodeAdapter[] vertices = G;
                private Object element = label;
                
                public Vertex[] getVertices() {
                    return vertices;
                }
                public Object getElement() { return element; }
                public void setElement(Object o) { element = o; }
                public String toString() { 
                    if (element != null)
                        return "Static " + element.toString(); 
                    else
                        return "Static Graph Adapter";
                }
            };
    }

    /** Return the title name of this adapter container. */
    public abstract String getName();

    public String toString() { return getName(); }

    public static Object getStructure(Object obj) {
        if (obj instanceof StaticList)
            return getStructure((StaticList)obj);

        if (obj instanceof StaticTree)
            return getStructure((StaticTree)obj);
        
        if (obj instanceof StaticGraph)
            return getStructure((StaticGraph)obj);

        Note.show(null, "Cannot adapt to " + obj);
        return null;
    }
}
