/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Key.java,v 1.12 2004/05/26 08:05:57 vkaravir Exp $ */

package matrix.structures.memory;

import matrix.structures.FDT.FDT;
import matrix.util.Note;

public class Key extends Element implements Loadable {
    public static final String EMPTY = "";
    public static final Key Empty = new Key(EMPTY);

    private String key;
    private Object object;

    public Key(String k) {
        key = (k != null) ? k : EMPTY;
    }

    public Key(Object o) {
        object = o;
        if (o != null)
            key = o.toString();
        else
            key = EMPTY;
    }

    public Key(int i) {
        key = String.valueOf(i);
    }

    Key orig = null;
    public Key getOrig() {
        return orig;
    }

    public Object getObject() { return object; }

    public void setOrig(Key orig) {
        this.orig = orig;
    }

    /* This method is invoked by VisualType each time this key is selected. */
    public void select() {
    }
    public boolean isSelected() {
	return false;
    }

    // FDT interface Methods
    //public FDT getNewStructure() { return new Key(null); }

    // operational interface (CDT) (not actually defined)
    public boolean equals(Key k) {
        if (this == k) return true;
        if (k != null) {
            return key.equals(k.getStr());
        } else return false;
    }
    public boolean equals(Object k) {
        if (k instanceof Key)
            return equals((Key)k);
        else if (k instanceof String) {
            if ((key == null) || (key == EMPTY)) return true;
            return key.equals((String)k);
        } else return false;
    }

    /*
    public boolean equals(Object k) {
        if (this == k) return true;
        if (k != null) {
            return key.equals(k.toString());
        } else return false;
    }
    */
    public String cmpString() {
        return key.toString();
    }

    public String getStr() { return key; }
    public void setKey(Key k) { key = k.getStr(); }

    public FDT getKey() { return this; }
    public void setKey(FDT s) {
        Note.out(this, "SetKey");
        if (s instanceof Key) {
            key = ((Key) s).getStr();
        } else Note.out(this, "Error: cant set non-key to key");
    }

    // Loadable
    public String toString() {
        return key;
    }
    public Loadable fromString(String repr) {
        return new Key(repr);
    }

    static final long serialVersionUID = 2497751812742138576L;
}
