/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VirtualBoolean.java,v 1.13 2004/05/26 08:05:58 vkaravir Exp $ */

package matrix.structures.memory;

import matrix.animation.Animator;

/**
 * Boolean class that works with matrix's animator. Substitute primitive booleans
 * with this when dealing with <code>Animator</code>.
 *
 * @see VirtualPrimitive
 *
 * @author Petri Tenhunen
 */
public class VirtualBoolean extends VirtualPrimitive {


    /**
     * Default constructor to animated VirtualBoolean with
     * false as initial value
     */
    public VirtualBoolean() {
        this(false);
    }

    /**
     * constructor with initial value and information if object
     * is animated or not
     *
     * @param value initial value
     * @param animated is object animated with matrix animator (usually true)
     */
    public VirtualBoolean(boolean value) {
        super();
        primitiveValue = new Boolean(value);
    }


    // CDT

    /**
     * Assign argument value to this object.
     * @param value new value to be assigned
     * @return new value of this object
     */
    public boolean assign(boolean value) {
        Boolean newValue = new Boolean(value);
        Animator.getActiveAnimator().putAssignment(this, newValue, 0);
        primitiveValue = newValue;
        return value;
    }

    /** Change value of this object to it's negation */
    public boolean toggle() {
        boolean i = eval();
        assign(!eval());
        return i;
    }

    /** Get primitive value of this object
     * @return primiteva value assigned to this object
     */
    public boolean eval() {
        return ((Boolean)primitiveValue).booleanValue();
    }


    static final long serialVersionUID = -2895426568916457306L;
}
