/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Function.java,v 1.6 2004/05/25 13:28:25 kstalnac Exp $ */

package matrix.util;

/**
 * A small hack to simulate first-class functions in Java. Please
 * override one or more of the f methods in concrete Functions.
 *
 * All parameters and return values are Objects (i.e., this is
 * dynamically typed), because of Java's primitive type system.
 *
 * @author Riku Saikkonen
 * @version $Revision: 1.6 $
 */
public abstract class Function {
    // Parameters to be set via the Function constructor, to work
    // around Java not including local variables in inner class
    // closures.
    public Object parameter, parameter2, parameter3;

    public Function() { }

    public Function(Object p) {
        parameter = p;
    }

    public Function(Object p1, Object p2) {
        parameter = p1;
        parameter2 = p2;
    }

    public Function(Object p1, Object p2, Object p3) {
        parameter = p1;
        parameter2 = p2;
        parameter3 = p3;
    }

    public Object f() {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    public Object f(Object o1) {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    public Object f(Object o1, Object o2) {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    public Object f(Object o1, Object o2, Object o3) {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    public Object f(Object o1, Object o2, Object o3, Object o4) {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    public Object f(Object o1, Object o2, Object o3, Object o4, Object o5) {
        Note.err(this, "Unimplemented Function.f");
        return null;
    }
    /*
     * Add versions of f with more parameters if you need them...
     *
     * "If you have a procedure with ten parameters, you probably
     * missed some."  -- Alan Perlis
     */
}
