/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Tranverse.java,v 1.6 2004/05/26 08:06:01 vkaravir Exp $ */

package matrix.util;
import java.util.Vector;

import matrix.structures.FDT.probe.BinaryTree;

/**
 * The Tranverse class contains methods to tranverse binary trees
 * It returns the result as a <code>String</code> containing the keys of
 * the tranversed nodes in the order in which they were tranversed,
 * except printTree, which prints to standard output
 * 
 * @author Jussi Nikander
 */
public class Tranverse {

    /** used to store the order in which the nodes have been tranversed */
    private static String order = "";

    /** The queue used by <code>tranverseLevelOrder</code> */
    private static Vector queue = new Vector(0, 1);

    /**
     * Tranverses a binary tree in level order
     * @param n The root node of the tree to be tranversed.
     * @return String containing the order in which the nodes were tranversed.
     */
    public static String tranverseLevelOrder(BinaryTree n) {

	order = ""; 
	queue.removeAllElements();
	translvl(n);
	return order;
    }

    /**
     * Tranverses a binary tree in preorder
     * @param n The root of the tree to be tranversed.
     * @return String containing the order in which the nodes were tranversed.
     */
    public static String tranversePreOrder(BinaryTree n) {

	order = "";
	transpre(n);
	return order;
    }

    /**
     * Tranverses a binary tree in inorder
     * @param n The root of the tree to be tranversed.
     * @return String containing the order in which the nodes were tranversed.
     */
    public static String tranverseInOrder(BinaryTree n) {

	order = "";
	transin(n);
	return order;
    }

    /** 
     * Tranverses a binary tree in postorder
     * @param n The root of the tree to be tranversed.
     * @return String containing the order in which the nodes were tranversed.
     */
    public static String tranversePostOrder(BinaryTree n) {
	
	order = "";
	transpost(n);
	return order;
    }

    /**
     * Prints a representation of a Binary tree. 
     * @param n The root of the tree to be tranversed.
     */
    public static int printTree(BinaryTree n) {
	if(n == null) {
	    System.out.print("Null");
	    return 0;
	}
	BinaryTree node;
	int nroOfNodes = 0;
	put(n);
	do {
	    node = (BinaryTree) get();
	    if(node != null) {
		if(node.getElement() == null)
		    System.out.print("Node: null, left child: ");
		else
		    System.out.print("Node: " + node.getElement().toString() + ", left child: ");
		if(node.getLeft() != null) {
		    if(node.getLeft().getElement() == null)
			System.out.print("null node");
		    else
			System.out.print(node.getLeft().getElement().toString() + " ");
		} else
		    System.out.print("null ");
		System.out.print("right child: ");
		if(node.getRight() != null) {
		    if(node.getRight().getElement() == null)
			System.out.print("null node ");
		    else
			System.out.println(node.getRight().getElement().toString());
		} else
		    System.out.println("null");
		put(node.getLeft());
		put(node.getRight());
		nroOfNodes++;
	    }	
	} while(!queue.isEmpty());
	return nroOfNodes;
    }

    /**
     * A private function used to tranverse a tree in level order
     * @param n The root of the tree to be tranversed.
     */
    private static void translvl(BinaryTree n) {
	BinaryTree node;
	put(n);
	do {
	    node = (BinaryTree) get();
	    if(node != null) {
		order = order + " " + node.getElement().toString();
		put(node.getLeft());
		put(node.getRight());
	    }
	} while(!queue.isEmpty());
    }

    /**
     * A private function used to tranverse a tree in preorder 
     * @param n The root of the subtree to be tranversed in current recursive call.
     */
    private static void transpre(BinaryTree n) {

	if(n != null) {
	    order = order + " " + n.getElement().toString();
	    transpre(n.getLeft());
	    transpre(n.getRight());
	}
    }

    /**
     * A private function used to tranverse a tree in inorder
     * @param n The root of the subtree to be tranversed in current recursive call.
     */
    private static void transin(BinaryTree n) {
	if(n != null) {
	    transin(n.getLeft());
	    order = order + " " + n.getElement().toString();
	    transin(n.getRight());
	}
    }

    /** 
     * A private function used to tranverse a tree in postorder
     * @param n The root of the subtree to be tranversed in current recursive call.
     */
    private static void transpost(BinaryTree n) {
	if(n != null) {
	    transpost(n.getLeft());
	    transpost(n.getRight());
	    order = order + " " + n.getElement().toString();
	}
    }

    /**
     * used to put items in queue
     * @param t The <code>Object</code> to be put.
     */
    private static void put(Object t) {
	if(t != null)
	    queue.addElement(t);
	return;
    }

    /**
     * used to get items from queue
     * @return the first object of the queue
     */
    private static Object get() {
	Object temp;
	if(!queue.isEmpty()) {
	    temp = queue.elementAt(0);
	    queue.removeElementAt(0);
	    return temp;
	}
	else
	    return null;
    }
}







