/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.*;

import matrix.structures.code.*;
import matrix.util.Note;

/**
 * This is the container for code representation.
 *
 * @author Ari Korhonen
 * @version $Revision: 1.12 $
 */
public class VisualCode extends VisualContainer {
    Code code;

    public VisualCode(Code code) {
        this.code = code;
        //super(code);
        setName(code.getCodeName());
    }
    
    public void insert(Object structure) {}

    public void validateComponents() {
        CodeLine[] codelines = code.getCodeLines();
        Component[] cmps = getItems();
        if ((cmps == null) || 
            (cmps.length == 0) || 
            (!(cmps[0] instanceof VisualTextComponent)))
            add(new VisualTextComponent(code));
        else
            ((VisualTextComponent)cmps[0]).setCode(code);
        for (int i=0; i<codelines.length; i++) {
            if (i+1 >= cmps.length) // missing visual component
                add(new VisualCodeLine(code, codelines[i]));
            else ((VisualCodeLine)cmps[i+1]).setLine(codelines[i]);
        }
        while (cmps.length > codelines.length+1) 
            removeItem(codelines.length+1);
    }

    public LayoutSize doActualLayout() {
        Component[] cmps = getItems();
        int newWidth = 0;
        int newHeight = 0;
        int x = 0;
        int y = 0;
        for (int i=0; i<cmps.length; i++) {
            if (!(cmps[i] instanceof VisualComponent))
                Note.err(this, "not a visual component " + cmps[i]);
            VisualComponent vcl = (VisualComponent)cmps[i];
            LayoutSize vclSize = vcl.lGetPreferredSize();
            if (vclSize.width > newWidth) newWidth = vclSize.width;
            newHeight += vclSize.height;
            vcl.lSetBounds(x,y,vclSize.width,vclSize.height);
            y += vclSize.height;
        }
        return new LayoutSize(newWidth, newHeight);
    }

    public void paint(Graphics g) {
        super.paint(g);
        if (isPicked()) return; // no frames
        paintFrame(g);
    }
}


