/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualList.java,v 1.34 2004/05/26 08:06:11 vkaravir Exp $ */

package matrix.visual;

import java.awt.Graphics;

import matrix.animation.StructureChangeOperation;
import matrix.structures.CDT.CDT;
import matrix.structures.FDT.*;
import matrix.structures.memory.Key;
import matrix.util.Note;

public class VisualList extends VisualContainer {
    LinkedList list;
    
    public VisualList(LinkedList l) {
        super(l);
        this.list = l;
    }
    
    public String getRepresentationName() { return "VisualList"; }

    // XXX doesn't seem to be used, remove?
    // (breaks things because the new component isn't in a container!)
    //public VisualComponent createNewVisualComponent(LinkedList key) {
    //    return new VisualListComponent(key);
    //}

    public boolean canInsert() {
	return true;
    }

    public void insert(Object structure) {
	if(getStructure() instanceof CDT) {
	    CDT cdt = (CDT)getStructure();
	    FDT tmp = cdt.insert(structure);
	    setList((LinkedList)tmp);
	}
	else if(list.getElement() == null) {
	    //if(structure instanceof Element)
	    if(structure instanceof FDT)
		list.setElement(structure);
	    else
		list.setElement(new Key(structure));
	    setList(list);
	}else {
	    LinkedList tmp = list.getNewNode(structure);
	    tmp.setNext(list);
	    setList(tmp);
	}
	/*
	LinkedList tmp = list.getNewNode(structure);
	tmp.setNext(list);
	setList(tmp);
	Note.out(this, "Insert done");
	*/
    }
    /*
    public void insertAt(int index) {
        list.setPos(index);
        list.insert(read.getStructure());
    }
    */

    public void delete(Object structure) {
	if (getStructure() instanceof CDT) {
	    CDT cdt = (CDT)getStructure();
	    FDT tmp = cdt.delete(structure);
	    setList((LinkedList)tmp);
	}
    }

	protected void removeKeyHelp(Object o) {
		if(getStructure() instanceof CDT) {
			delete(o);
		}
		else
			super.removeKeyHelp(o);	    
	}

   
    public FDT getStructure() {
        return list;
    }

    public void setStructure(FDT fdt) {
	if(fdt instanceof LinkedList)
	    list = (LinkedList)fdt;
    }

    protected void setList(LinkedList newList) {
	LinkedList oldList = (LinkedList)getStructure();
	getAnimator().put(new StructureChangeOperation(oldList, newList));
	setStructure(newList);
    }

    // -=- validation -=-

    /*    
    public FDT currValue() {
        Object o = list.currValue();
        if (o instanceof FDT) 
            return (FDT)o;
        else {
	  //Note.out(this, "Remember to implement VisualType.getKey() properly!");
            return turnFDT(o);
        }
    }
    */

    private VisualListComponent createNewVisualListComponent(LinkedList curr, 
							     int index) {
	VisualListComponent tmp = new VisualListComponent(curr);
	tmp.setIndex(index);
	add(tmp, index);
	return tmp;
    }

    private VisualListComponent moveVisualListComponent(int from, 
							int to) {
	VisualListComponent tmp = (VisualListComponent)getItem(from);
	moveItem(from, to);
	tmp.setIndex(to);
	return tmp;
    }						
    
    private void validateReference(int index, 
				   VisualListComponent from, 
				   VisualListComponent to) {
	
	from.setNext(to);
	to.setPrevious(from);
	int n = getLinkCount();
	VisualReference tmp;
	if(n > index) {
	    tmp = getLink(index);
	    tmp.setSource(from);
	    tmp.setTarget(to);
	}
	else {
	    tmp = new VisualReference(from, to);
	    add(tmp, index);
	}
    } 

    public void validateComponents() {
	int index = 0;
	VisualListComponent curr = null, prev = null;
	LinkedList lst = list;
	while (lst != null /*&& lst.getElement() != null*/) {
	    //Note.out(this, "loop");
	    int k = lookUpVisualItem(lst, index);
	    //Note.out(this, k + ", " + getItemCount());
	    if (k == index) { // found and valid
		curr = (VisualListComponent) getItem(index); // allready validated
	    } else if (k < 0) { // not found
		curr = createNewVisualListComponent(lst, index);
	    } else { // k > index, i.e. found but not valid
		curr = moveVisualListComponent(k, index);
	    }
	    if (prev != null)
		validateReference(index-1, prev, curr);
	    if (curr.isDuplicate()) {
		Note.out(this, "duplicate!");
		break; // end of validation if duplicate encountered
	    }
	    prev = curr;
	    index++;
	    lst = lst.getNext();
	}
	// remove unused components. Currently not working as it should?
	//Note.out(this, "index: " + index + " items: " + getItemCount() + " links: " + getLinkCount());
	while(index < getItemCount())
	    removeItem(index);
	if(index > 1) {
	    while(index <= getLinkCount())
		removeLink(index-1);
	}
    }

    // -=- layout -=-

    public int getGapWidth() { return 10; }
    public int getGapHeight() { return getGapWidth(); }

    public LayoutSize doActualLayout() {
        //Note.out(this, "doActualLayout "+list);
        if (list == null)
            return new LayoutSize(40, 20); // size of empty list
        return getListSize((VisualListComponent)getItem(0),
                           new LayoutBounds(0, 0, 0, 0));
    }

    /* This is the actual method to recursively count the size of the list and
       lay out its representation. */
    private LayoutSize getListSize(VisualListComponent vtc,
                                   LayoutBounds bounds) {
        // x + empty space in left
        int x = bounds.x + getGapWidth();
        int y = bounds.y;
        bounds.width = 0;
        bounds.height = 0;
        while (vtc != null) {
            LayoutSize vtcSize = vtc.lGetPreferredSize();
            vtc.lSetBounds(x, y, vtcSize.width, vtcSize.height);
            x += vtcSize.width + getGapWidth();
            bounds = bounds.union(vtc.lGetBounds());
            vtc = vtc.getNext();
        }
        // return bounds + empty space in right
        return new LayoutSize(bounds.width+getGapWidth(), bounds.height);
    }


    public void paint(Graphics g) {
        super.paint(g);
        if (isMinimized()) { paint_minimized(g); return; }
        if (isPicked()) return; // no frames
        paintFrame(g);
    }
}







