/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.PopupMenu;

import matrix.structures.FDT.FDT;
import matrix.util.Note;

/** VisualMenuHotSpot is an abstraction for popUpMenu visualization
 * @see VisualContainer
 *
 * @author Ari Korhonen
 * @version $Revision: 1.10 $
 */
public class VisualMenuHotSpot extends VisualHotSpot {

    // --- constructors ---

    /** Note: The parent of this visual component should always be another visual component. */
    public VisualMenuHotSpot(FDT fdt) {
        super(fdt);
        setLabel("[^]");
    }
    
    protected void removePopupMenu() {
	// do nothing because we share the visual hot spot created by visual container
    }

    protected PopupMenu getPopUp() {
	return ((VisualType)getParent()).getPopUp();
    }

    protected void setPopUp(PopupMenu pm) {
	Note.err(this, "Cannot change popUp for " + this);
    }

    public void selectEntity(boolean isShiftDown) {
        if (getPopUp() != null && popUpEnabled()) getPopUp().show(this, -10, -10);
    }
}
