/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import matrix.structures.FDT.*;
import matrix.util.Note;

/** VisualReziseHotSpot is an abstraction for special active resize components within containers.
 * @see VisualContainer
 *
 * @author Ari Korhonen
 * @version $Revision: 1.6 $
 */
public class VisualResizeHotSpot extends VisualHotSpot {

    // --- constructors ---

    public VisualResizeHotSpot(FDT fdt) {
        super(fdt);
        setLabel("->");
    }

    public Array getArray() {
        if (getStructure() instanceof Array) {
            return (Array)getStructure();
        } else Note.show(this, "Cannot resize this kind of structure (not an Array)");
        return null;
    }

    int slotSize;
    int pick_x;
    public void pick(int x, int y) {
        picked = true;
        Array a = getArray();
        if (a != null) {
            slotSize = getBounds().width;
            pick_x = x;
        }
    }

    public void drag(int x, int y) {
        Array a = getArray();
        if ((a != null) && (slotSize > 0)) {
            if (x > pick_x + slotSize/2) {
                a.setLast(a.getLast()+2);
                setInvalid();
            }
            else if (x < pick_x) {
                if (a.getLast() - a.getFirst() > 0)
                    a.setLast(a.getLast()-1);
                setInvalid();
            }
        }
    }

    public void released() {
        picked = false;
        setInvalid();
    }
}
