/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package prototype.ui;

import java.awt.*;
import java.awt.event.*;

import content.exercises.Judgement;

public class GradeDialog extends Dialog {

    public static final int ROWS = 10;
    public static final int COLUMNS = 50;
    
    Judgement judgement;

    protected TextArea text;
    protected Panel buttonPanel;

    public GradeDialog() {
        super(new Frame(), "Grade");
        setLayout(new GridBagLayout());

	text = makeTextArea();

        GridBagConstraints gbc = new GridBagConstraints();
	gbc.insets = new Insets(10,10,10,10);
	gbc.fill=GridBagConstraints.BOTH;
	gbc.anchor=GridBagConstraints.NORTH;
	gbc.weighty=1.0;

        add(text, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
	gbc.weighty=0.0;

        add(buttonPanel=makeButtonPanel(), gbc);

        setModal(true);

        // avctivate window close button
        this.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) { close(); }
            });

        pack();
    }



  protected TextArea makeTextArea() {

	TextArea t = new TextArea("", ROWS, COLUMNS);
	t.enableInputMethods(false);
        t.setEditable(false);
	return t;
    }

    protected Panel makeButtonPanel() {
        Panel buttonPanel = new Panel();
        buttonPanel.setLayout(new FlowLayout());

        Button contButton = new Button("Close");
        contButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    close();
                }
            });
        buttonPanel.add(contButton);
        return buttonPanel;
    }



    public GradeDialog(Judgement j) {
        this();
        judgement = j;
        String[] names = j.getNames();
        int[] points = j.getPoints();
        int max = j.getMax();
        StringBuffer buf = new StringBuffer();
	if (max != 1) {
	    buf.append(sum(points));
	    buf.append(" steps out of "+ max + " correct.");
	    buf.append("\n");
	} else if (sum(points) == 1)
	    buf.append("Exercise correct!\n");
	else 
	    buf.append("Exercise incorrect!\n");
	
	/*
	if (j.isComplete())
            buf.append("You have reached the final state.\n");
	*/

	text.setText(buf.toString());
	pack();
    }

    
    protected void close() {
        dispose();
    }
    
    
    protected Judgement getJudgement() {
        return judgement;
    }
    

    static int sum(int[] a) {
	int val = 0;
	for (int i = 0; i < a.length; i++) {
	    val = val + a[i];
	}
	return val;
    }

}
